/*
 * This file is part of xsoundtrack - a sound tracker for Linux.
 * Sed - july august september 1998.
 * This is total free software.
 *-------------------------------------
 * the instrument editor - drawing functions + handling of moving (handling of editing is in sub_instrument.c)
 */

#include "instrument.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/keysym.h>

#include "interface.h"
#include "xm.h"
#include "screen.h"
#include "sub_instrument.h"

void init_instrument_pixmap(INTERFACE *i, instrument *inst)
{
  /* not all is inited here, for example the box necessary for string-drawing are not drawn, they are drawn
   * with the string, so we don't have to draw them here again */
  /* volume envelope drawing */
  draw_3d_box(i->X_ecran, inst->pixmap, i->context, 4, 4, 325+8+12, 65*2+8+12+3+3, GREY, UP_3D);
  draw_string(i->X_ecran, inst->pixmap, i->context, 8+6*19, 16, "Volume envelope", i->little, BLACK);
  draw_string(i->X_ecran, inst->pixmap, i->context, 7, 16+13+3, "64", i->little, BLUE);
  draw_string(i->X_ecran, inst->pixmap, i->context, 7+6, 16+65*2+2+3, "0", i->little, BLUE);
  /* use volume envelope... */
  draw_3d_box(i->X_ecran, inst->pixmap, i->context, 4+3*5, 65*2+8+21+3+2, 325+8+12-6*5, 18, GREY, UP_3D);
  draw_3d_box(i->X_ecran, inst->pixmap, i->context, 3*5+2*6, 65*2+8+21+3+2+4, 10, 10, LIGHTBLUE, UP_3D);
  draw_3d_box(i->X_ecran, inst->pixmap, i->context, 3*5+2*6+24*6, 65*2+8+21+3+2+4, 10, 10, LIGHTBLUE, UP_3D);
  draw_3d_box(i->X_ecran, inst->pixmap, i->context, 3*5+2*6+40*6, 65*2+8+21+3+2+4, 10, 10, LIGHTBLUE, UP_3D);
  draw_string(i->X_ecran, inst->pixmap, i->context, 7+6*3+3*5, 65*2+8+21+3+2+12,
	      "Use volume envelope     Use sustain     Use loop", i->little, BLACK);
  /* panning envelope drawing */
  draw_3d_box(i->X_ecran, inst->pixmap, i->context, 4, PINY+4, 325+8+12, 64*2+8+12+3+3, GREY, UP_3D);
  draw_string(i->X_ecran, inst->pixmap, i->context, 8+6*19-3, PINY+16, "Panning envelope", i->little, BLACK);
  draw_string(i->X_ecran, inst->pixmap, i->context, 7, PINY+16+13+3, "Ri", i->little, BLUE);
  draw_string(i->X_ecran, inst->pixmap, i->context, 7, PINY+16+64*2+2+3, "Le", i->little, BLUE);
  /* use panning envelope... */
  draw_3d_box(i->X_ecran, inst->pixmap, i->context, 4+3*5-3, PINY-2+65*2+8+21+3+2, 325+8+12-6*5+6, 18, GREY, UP_3D);
  draw_3d_box(i->X_ecran, inst->pixmap, i->context, 3*5+2*6-3, PINY-2+65*2+8+21+3+2+4, 10, 10, LIGHTBLUE, UP_3D);
  draw_3d_box(i->X_ecran, inst->pixmap, i->context, 3*5+2*6+24*6+3, PINY-2+65*2+8+21+3+2+4, 10, 10, LIGHTBLUE, UP_3D);
  draw_3d_box(i->X_ecran, inst->pixmap, i->context, 3*5+2*6+40*6+3, PINY-2+65*2+8+21+3+2+4, 10, 10, LIGHTBLUE, UP_3D);
  draw_string(i->X_ecran, inst->pixmap, i->context, 7+6*3+3*5-3, PINY-2+65*2+8+21+3+2+12,
	      "Use panning envelope     Use sustain     Use loop", i->little, BLACK);
  /* nb points... (for volume) */
  draw_3d_box(i->X_ecran, inst->pixmap, i->context, NIX+5, 5+4, 148-6*9-6, 74+71, GREY, UP_3D);
  draw_string(i->X_ecran, inst->pixmap, i->context, NIX+9+6*5, 5+1+4+4+8+11-5, "Nb", i->little, BLACK);
  draw_string(i->X_ecran, inst->pixmap, i->context, NIX+9+6, 5+1+4+4+8+11+5, "points", i->little, BLACK);
  draw_string(i->X_ecran, inst->pixmap, i->context, NIX+9, 5+1+4+4+13+3+13+6+8+11-5, "Sustain", i->little, BLACK);
  draw_string(i->X_ecran, inst->pixmap, i->context, NIX+9+6*2, 5+1+4+4+13+3+13+6+8+11+5, "point", i->little, BLACK);
  draw_string(i->X_ecran, inst->pixmap, i->context, NIX+9+6*3, 5+9+13+3+13+6+8+11-5+13+3+13+6, "Loop", i->little, BLACK);
  draw_string(i->X_ecran, inst->pixmap, i->context, NIX+9+6*2, 5+9+13+3+13+6+8+11+5+13+3+13+6, "start", i->little, BLACK);
  draw_string(i->X_ecran, inst->pixmap, i->context, NIX+9+6*3, 5+1+4+4+(13+3+13+6)*3+8+11-5, "Loop", i->little, BLACK);
  draw_string(i->X_ecran, inst->pixmap, i->context, NIX+9+6*4, 5+1+4+4+(13+3+13+6)*3+8+11+5, "end", i->little, BLACK);
  draw_arrow(i->X_ecran, inst->pixmap, i->context, NIX+9+6*7+5, 5+1+4+4, UP_3D, ARROW_UP);
  draw_arrow(i->X_ecran, inst->pixmap, i->context, NIX+9+6*7+5, 5+1+4+4+13+3, UP_3D, ARROW_DOWN);
  draw_arrow(i->X_ecran, inst->pixmap, i->context, NIX+9+6*7+5, 5+1+4+4+13+3+13+6, UP_3D, ARROW_UP);
  draw_arrow(i->X_ecran, inst->pixmap, i->context, NIX+9+6*7+5, 5+1+4+4+13+3+13+6+13+3, UP_3D, ARROW_DOWN);
  draw_arrow(i->X_ecran, inst->pixmap, i->context, NIX+9+6*7+5, 5+1+4+4+(13+3+13+6)*2, UP_3D, ARROW_UP);
  draw_arrow(i->X_ecran, inst->pixmap, i->context, NIX+9+6*7+5, 5+1+4+4+(13+3+13+6)*2+13+3, UP_3D, ARROW_DOWN);
  draw_arrow(i->X_ecran, inst->pixmap, i->context, NIX+9+6*7+5, 5+1+4+4+(13+3+13+6)*3, UP_3D, ARROW_UP);
  draw_arrow(i->X_ecran, inst->pixmap, i->context, NIX+9+6*7+5, 5+1+4+4+(13+3+13+6)*3+13+3, UP_3D, ARROW_DOWN);
  /* nb points... (for panning) */
  draw_3d_box(i->X_ecran, inst->pixmap, i->context, NIX+5, 5+PINY+4, 148-6*9-6, 74+71, GREY, UP_3D);
  draw_string(i->X_ecran, inst->pixmap, i->context, NIX+9+6*5, 5+PINY+1+4+4+8+11-5, "Nb", i->little, BLACK);
  draw_string(i->X_ecran, inst->pixmap, i->context, NIX+9+6, 5+PINY+1+4+4+8+11+5, "points", i->little, BLACK);
  draw_string(i->X_ecran, inst->pixmap, i->context, NIX+9, 5+PINY+1+4+4+13+3+13+6+8+11-5, "Sustain", i->little, BLACK);
  draw_string(i->X_ecran, inst->pixmap, i->context, NIX+9+6*2, 5+PINY+1+4+4+13+3+13+6+8+11+5, "point", i->little, BLACK);
  draw_string(i->X_ecran, inst->pixmap, i->context, NIX+9+6*3, 5+PINY+9+16+13+6+8+11-5+16+19, "Loop", i->little, BLACK);
  draw_string(i->X_ecran, inst->pixmap, i->context, NIX+9+6*2, 5+PINY+9+13+3+13+6+8+16+16+19, "start", i->little, BLACK);
  draw_string(i->X_ecran, inst->pixmap, i->context, NIX+9+6*3, 5+PINY+9+(13+3+13+6)*3+8+11-5, "Loop", i->little, BLACK);
  draw_string(i->X_ecran, inst->pixmap, i->context, NIX+9+6*4, 5+PINY+9+(13+3+13+6)*3+8+11+5, "end", i->little, BLACK);
  draw_arrow(i->X_ecran, inst->pixmap, i->context, NIX+9+6*7+5, 5+PINY+1+4+4, UP_3D, ARROW_UP);
  draw_arrow(i->X_ecran, inst->pixmap, i->context, NIX+9+6*7+5, 5+PINY+1+4+4+13+3, UP_3D, ARROW_DOWN);
  draw_arrow(i->X_ecran, inst->pixmap, i->context, NIX+9+6*7+5, 5+PINY+1+4+4+13+3+13+6, UP_3D, ARROW_UP);
  draw_arrow(i->X_ecran, inst->pixmap, i->context, NIX+9+6*7+5, 5+PINY+1+4+4+13+3+13+6+13+3, UP_3D, ARROW_DOWN);
  draw_arrow(i->X_ecran, inst->pixmap, i->context, NIX+9+6*7+5, 5+PINY+1+4+4+(13+3+13+6)*2, UP_3D, ARROW_UP);
  draw_arrow(i->X_ecran, inst->pixmap, i->context, NIX+9+6*7+5, 5+PINY+1+4+4+(13+3+13+6)*2+13+3, UP_3D, ARROW_DOWN);
  draw_arrow(i->X_ecran, inst->pixmap, i->context, NIX+9+6*7+5, 5+PINY+1+4+4+(13+3+13+6)*3, UP_3D, ARROW_UP);
  draw_arrow(i->X_ecran, inst->pixmap, i->context, NIX+9+6*7+5, 5+PINY+1+4+4+(13+3+13+6)*3+13+3, UP_3D, ARROW_DOWN);
  /* volume fadeout speed, ... */
  draw_3d_box(i->X_ecran, inst->pixmap, i->context, VFX+5, 5+VFY+4, 148-6*9, 74+71+(13+3+13+6)*3, GREY, UP_3D);
  draw_string(i->X_ecran, inst->pixmap, i->context, VFX+9+6, VFY+1+4+4+8+11-5, "Volume", i->little, BLACK);
  draw_string(i->X_ecran, inst->pixmap, i->context, VFX+9, 5+5+VFY+1+4+4+8+11-5, "Fadeout", i->little, BLACK);
  draw_string(i->X_ecran, inst->pixmap, i->context, VFX+9+6*2, 5+5+VFY+1+4+4+8+11+5, "speed", i->little, BLACK);
  draw_string(i->X_ecran, inst->pixmap, i->context, VFX+9, 5+VFY+9+(13+3+13+6)+14, "Vibrato type :", i->little, BLACK);
  draw_string(i->X_ecran, inst->pixmap, i->context, VFX+9, 5+VFY+1+4+4+(13+3+13+6)*4+8+11-5, "Vibrato", i->little, BLACK);
  draw_string(i->X_ecran, inst->pixmap, i->context, VFX+9+6*2, 5+VFY+1+4+4+(13+3+13+6)*4+8+11+5, "speed", i->little, BLACK);
  draw_string(i->X_ecran, inst->pixmap, i->context, VFX+9, 5+VFY+9+(16+13+6)*5+8+11-5, "Vibrato", i->little, BLACK);
  draw_string(i->X_ecran, inst->pixmap, i->context, VFX+9+6*2, 5+VFY+9+(13+3+13+6)*5+8+16, "sweep", i->little, BLACK);
  draw_string(i->X_ecran, inst->pixmap, i->context, VFX+9, 5+VFY+9+(13+3+13+6)*6+8+11-5, "Vibrato", i->little, BLACK);
  draw_string(i->X_ecran, inst->pixmap, i->context, VFX+9+6*2, 5+VFY+9+(13+3+13+6)*6+8+11+5, "depth", i->little, BLACK);
  draw_arrow(i->X_ecran, inst->pixmap, i->context, VFX+9+6*7+5, 5+VFY+1+4+4, UP_3D, ARROW_UP);
  draw_arrow(i->X_ecran, inst->pixmap, i->context, VFX+9+6*7+5, 5+VFY+1+4+4+13+3, UP_3D, ARROW_DOWN);
  draw_arrow(i->X_ecran, inst->pixmap, i->context, VFX+9+6*7+5, 5+VFY+1+4+4+(13+3+13+6)*4, UP_3D, ARROW_UP);
  draw_arrow(i->X_ecran, inst->pixmap, i->context, VFX+9+6*7+5, 5+VFY+1+4+4+(13+3+13+6)*4+13+3, UP_3D, ARROW_DOWN);
  draw_arrow(i->X_ecran, inst->pixmap, i->context, VFX+9+6*7+5, 5+VFY+1+4+4+(13+3+13+6)*5, UP_3D, ARROW_UP);
  draw_arrow(i->X_ecran, inst->pixmap, i->context, VFX+9+6*7+5, 5+VFY+1+4+4+(13+3+13+6)*5+13+3, UP_3D, ARROW_DOWN);
  draw_arrow(i->X_ecran, inst->pixmap, i->context, VFX+9+6*7+5, 5+VFY+1+4+4+(13+3+13+6)*6, UP_3D, ARROW_UP);
  draw_arrow(i->X_ecran, inst->pixmap, i->context, VFX+9+6*7+5, 5+VFY+1+4+4+(13+3+13+6)*6+13+3, UP_3D, ARROW_DOWN);
  draw_3d_box(i->X_ecran, inst->pixmap, i->context, VFX+50, VFY+74, 10, 10, LIGHTBLUE, UP_3D);
  draw_3d_box(i->X_ecran, inst->pixmap, i->context, VFX+50, VFY+74+20, 10, 10, LIGHTBLUE, UP_3D);
  draw_3d_box(i->X_ecran, inst->pixmap, i->context, VFX+50, VFY+74+20*2, 10, 10, LIGHTBLUE, UP_3D);
  draw_3d_box(i->X_ecran, inst->pixmap, i->context, VFX+50, VFY+74+20*3, 10, 10, LIGHTBLUE, UP_3D);
  /* the sine-form */
  {
    int j;
    static int pos[22][2]={
      {0, 5}, {0, 4}, {0, 3}, {1, 2}, {1, 1}, {2, 0}, {3, 1}, {3, 2}, {4, 3}, {4, 4}, {4, 5},
      {5, 6}, {5, 7}, {5, 8}, {6, 9}, {6, 10}, {7, 11}, {8, 10}, {8, 9}, {9, 8}, {9, 7}, {9, 6}
    };
    for(j=0; j<22; j++)
      draw_plot(i->X_ecran, inst->pixmap, i->context, pos[j][0]+VFX+30, pos[j][1]+67, 1, BLACK);
  }
  /* the square-form */
  {
    int j;
    static int pos[30][2]={
      {0, 5}, {0, 4}, {0, 3}, {0, 2}, {0, 1}, {0, 0}, {1, 0}, {2, 0}, {3, 0}, {4, 0}, {4, 1},
      {4, 2}, {4, 3}, {4, 4}, {4, 5}, {4, 6}, {4, 7}, {4, 8}, {4, 9}, {4, 10}, {4, 11}, {5, 11},
      {6, 11}, {7, 11}, {8, 11}, {8, 10}, {8, 9}, {8, 8}, {8, 7}, {8, 6}
    };
    for(j=0; j<30; j++)
      draw_plot(i->X_ecran, inst->pixmap, i->context, pos[j][0]+VFX+30, pos[j][1]+67+20, 1, BLACK);
  }
  /* the bizarre-forms */
  {
    int j;
    static int pos[22][2]={
      {0, 6}, {1, 7}, {2, 8}, {3, 9}, {4, 10}, {5, 11}, {5, 10}, {5, 9}, {5, 8}, {5, 7}, {5, 6},
      {5, 5}, {5, 4}, {5, 3}, {5, 2}, {5, 1}, {5, 0}, {6, 1}, {7, 2}, {8, 3}, {9, 4}, {10, 5}
    };
    for(j=0; j<22; j++)
      draw_plot(i->X_ecran, inst->pixmap, i->context, pos[j][0]+VFX+30-1, pos[j][1]+67+40, 1, BLACK),
      draw_plot(i->X_ecran, inst->pixmap, i->context, 10-pos[j][0]+VFX+30-1, pos[j][1]+67+60, 1, BLACK);
  }
  /* the close button */
  draw_3d_box(i->X_ecran, inst->pixmap, i->context, 470, 280, 8*5+4+3, 18, 3, UP_3D);
  draw_string(i->X_ecran, inst->pixmap, i->context, 473+6, 280+12, "Close", i->little, 0);
  /* the keyboard (not all, rest in a following fun (draw_keyboar) */
  draw_3d_box(i->X_ecran, inst->pixmap, i->context, (9*7*8+16+23-(9*8*7+8-1))/2, 366, 9*8*7+8-1, 28, GREY, UP_3D);
  draw_box(i->X_ecran, inst->pixmap, i->context, (9*7*8+16+23-(9*8*7+8-1))/2+4, 366+4, 9*8*7+8-8-1, 20, WHITE);
  {
    int j;
    for (j=0; j<8*7-1; j++)
      draw_line(i->X_ecran, inst->pixmap, i->context, 9*j+19+9, 366+4, 9*j+19+9, 366+4+20, GREY);
  }
}

void draw_volume_envelope(INTERFACE *i, instrument *inst)
{
  int j;
  draw_rectangle(i->X_ecran, inst->pixmap, i->context, 8+12, 20+3-2, 325-1, 65*2+2, 3, GREY);
  draw_box(i->X_ecran, inst->pixmap, i->context, 8+12, 20+3, 325, 65*2-1, LIGHTBLUE);
  /* if no points, then we don't show anything */
  if (!inst->nb_volume_points)
    return;
  /* draw the points */
  for (j=0; j<inst->nb_volume_points; j++)
    draw_plot(i->X_ecran, inst->pixmap, i->context, inst->volume_envelop[j][0]+8+12-1,
	      (64-inst->volume_envelop[j][1])*2+20+3-1, 3, BLUE);
  /* line them */
  for (j=0; j<inst->nb_volume_points-1; j++)
    draw_line(i->X_ecran, inst->pixmap, i->context,
	      inst->volume_envelop[j][0]+8+12, (64-inst->volume_envelop[j][1])*2+20+3,
	      inst->volume_envelop[j+1][0]+8+12, (64-inst->volume_envelop[j+1][1])*2+20+3, BLUE);
  /* now put the sustain point */
  if (inst->volume_type & 2) {
    int x=inst->volume_envelop[(int)inst->volume_sustain_point][0];
    draw_line(i->X_ecran, inst->pixmap, i->context, x+8+12, 20+3-3, x+8+12, 20+3+65*2-1+3, WHITE);
    draw_line(i->X_ecran, inst->pixmap, i->context, x+8+12-1, 20+3-1, x+8+12+2, 20+3-1, WHITE);
    draw_line(i->X_ecran, inst->pixmap, i->context, x+8+12-1, 20+3+65*2-1, x+8+12+2, 20+3+65*2-1, WHITE);
    draw_line(i->X_ecran, inst->pixmap, i->context, x+8+12-1, 20+3-2, x+8+12+2, 20+3-2, WHITE);
    draw_line(i->X_ecran, inst->pixmap, i->context, x+8+12-1, 20+3+65*2, x+8+12+2, 20+3+65*2, WHITE);
  }
  /* loop end, loop start */
  if (inst->volume_type & 4) {
    int x1=inst->volume_envelop[(int)inst->volume_loop_start][0];
    int x2=inst->volume_envelop[(int)inst->volume_loop_end][0];
    /* start */
    draw_line(i->X_ecran, inst->pixmap, i->context, x1+8+12, 20+3, x1+8+12, 20+3+65*2+2, RED);
    draw_line(i->X_ecran, inst->pixmap, i->context, x1+8+12-1, 20+3+65*2-1, x1+8+12+2, 20+3+65*2-1, RED);
    draw_line(i->X_ecran, inst->pixmap, i->context, x1+8+12-1, 20+3+65*2, x1+8+12+2, 20+3+65*2, RED);
    /* end */
    draw_line(i->X_ecran, inst->pixmap, i->context, x2+8+12, 20+3-3, x2+8+12, 20+3+65*2-1, RED);
    draw_line(i->X_ecran, inst->pixmap, i->context, x2+8+12-1, 20+3-1, x2+8+12+2, 20+3-1, RED);
    draw_line(i->X_ecran, inst->pixmap, i->context, x2+8+12-1, 20+3-2, x2+8+12+2, 20+3-2, RED);
  }
  /* if we are edited, draw in yellow the current point */
  if (inst->mode==EDIT && inst->subwindow==0)
    draw_plot(i->X_ecran, inst->pixmap, i->context, inst->volume_envelop[inst->cur_pos_0][0]+8+12-1,
	      (64-inst->volume_envelop[inst->cur_pos_0][1])*2+20+3-1, 3, YELLOW);
}

void draw_panning_envelope(INTERFACE *i, instrument *inst)
{
  int j;
  draw_rectangle(i->X_ecran, inst->pixmap, i->context, 8+12, PINY+20+3-2, 325-1, 64*2+2, 3, GREY);
  draw_box(i->X_ecran, inst->pixmap, i->context, 8+12, PINY+20+3, 325, 64*2-1, LIGHTBLUE);
  draw_line(i->X_ecran, inst->pixmap, i->context, 8+12,PINY+20+3+31*2, 8+12+325, PINY+20+3+31*2, GREY);

  /* if no points, then we don't show anything */
  if (!inst->nb_panning_points)
    return;
  /* draw the points */
  for (j=0; j<inst->nb_panning_points; j++)
    draw_plot(i->X_ecran, inst->pixmap, i->context, inst->panning_envelop[j][0]+8+12-1,
	      PINY+(63-inst->panning_envelop[j][1])*2+20+3-1, 3, BLUE);
  /* line them */
  for (j=0; j<inst->nb_panning_points-1; j++)
    draw_line(i->X_ecran, inst->pixmap, i->context,
	      inst->panning_envelop[j][0]+8+12, PINY+(63-inst->panning_envelop[j][1])*2+20+3,
	      inst->panning_envelop[j+1][0]+8+12, PINY+(63-inst->panning_envelop[j+1][1])*2+20+3, BLUE);
  /* now put the sustain point */
  if (inst->panning_type & 2) {
    int x=inst->panning_envelop[(int)inst->panning_sustain_point][0];
    draw_line(i->X_ecran, inst->pixmap, i->context, x+8+12, PINY+20+3-3, x+8+12, PINY+20+3+64*2-1+3, WHITE);
    draw_line(i->X_ecran, inst->pixmap, i->context, x+8+12-1, PINY+20+3-1, x+8+12+2, PINY+20+3-1, WHITE);
    draw_line(i->X_ecran, inst->pixmap, i->context, x+8+12-1, PINY+20+3+64*2-1, x+8+12+2, PINY+20+3+64*2-1, WHITE);
    draw_line(i->X_ecran, inst->pixmap, i->context, x+8+12-1, PINY+20+3-2, x+8+12+2, PINY+20+3-2, WHITE);
    draw_line(i->X_ecran, inst->pixmap, i->context, x+8+12-1, PINY+20+3+64*2, x+8+12+2, PINY+20+3+64*2, WHITE);
  }
  /* loop end, loop start */
  if (inst->panning_type & 4) {
    int x1=inst->panning_envelop[(int)inst->panning_loop_start][0];
    int x2=inst->panning_envelop[(int)inst->panning_loop_end][0];
    /* start */
    draw_line(i->X_ecran, inst->pixmap, i->context, x1+8+12, PINY+20+3, x1+8+12, PINY+20+3+64*2+2, RED);
    draw_line(i->X_ecran, inst->pixmap, i->context, x1+8+12-1, PINY+20+3+64*2-1, x1+8+12+2, PINY+20+3+64*2-1, RED);
    draw_line(i->X_ecran, inst->pixmap, i->context, x1+8+12-1, PINY+20+3+64*2, x1+8+12+2, PINY+20+3+64*2, RED);
    /* end */
    draw_line(i->X_ecran, inst->pixmap, i->context, x2+8+12, PINY+20+3-3, x2+8+12, PINY+20+3+64*2-1, RED);
    draw_line(i->X_ecran, inst->pixmap, i->context, x2+8+12-1, PINY+20+3-1, x2+8+12+2, PINY+20+3-1, RED);
    draw_line(i->X_ecran, inst->pixmap, i->context, x2+8+12-1, PINY+20+3-2, x2+8+12+2, PINY+20+3-2, RED);
  }
  /* if we are edited, draw in yellow the current point */
  if (inst->mode==EDIT && inst->subwindow==3)
    draw_plot(i->X_ecran, inst->pixmap, i->context, inst->panning_envelop[inst->cur_pos_3][0]+8+12-1,
	      PINY+(63-inst->panning_envelop[inst->cur_pos_3][1])*2+20+3-1, 3, YELLOW);
}

void draw_nb_points_volume(INTERFACE *i, instrument *inst)
{
  char s[5];		/* don't be too happy of this static char, no buffer overflow is possible */
  draw_box(i->X_ecran, inst->pixmap, i->context, NIX+9+6*7+5+13+3, 5+1+4+4+8, 6*6-4-16, 12, LIGHTBLUE);
  sprintf(s, "%2d", inst->nb_volume_points);
  draw_string(i->X_ecran, inst->pixmap, i->context, NIX+9+6*7+5+13+3+2, 5+1+4+4+8+10, s, i->little, BLUE);
}

void draw_sustain_point_volume(INTERFACE *i, instrument *inst)
{
  char s[5];
  draw_box(i->X_ecran, inst->pixmap, i->context, NIX+9+6*7+5+13+3, 5+1+4+4+13+3+13+6+8, 6*6-4-16, 12, LIGHTBLUE);
  sprintf(s, "%2d", inst->volume_sustain_point);
  draw_string(i->X_ecran, inst->pixmap, i->context, NIX+9+6*7+5+13+3+2, 5+1+4+4+13+3+13+6+8+10, s, i->little, BLUE);
}

void draw_loop_start_volume(INTERFACE *i, instrument *inst)
{
  char s[5];
  draw_box(i->X_ecran, inst->pixmap, i->context, NIX+9+6*7+5+13+3, 5+1+4+4+(13+3+13+6)*2+8, 6*6-4-16, 12, LIGHTBLUE);
  sprintf(s, "%2d", inst->volume_loop_start);
  draw_string(i->X_ecran, inst->pixmap, i->context, NIX+9+6*7+5+13+3+2, 5+1+4+4+(13+3+13+6)*2+8+10, s, i->little, BLUE);
}

void draw_loop_end_volume(INTERFACE *i, instrument *inst)
{
  char s[5];
  draw_box(i->X_ecran, inst->pixmap, i->context, NIX+9+6*7+5+13+3, 5+1+4+4+(13+3+13+6)*3+8, 6*6-4-16, 12, LIGHTBLUE);
  sprintf(s, "%2d", inst->volume_loop_end);
  draw_string(i->X_ecran, inst->pixmap, i->context, NIX+9+6*7+5+13+3+2, 5+1+4+4+(13+3+13+6)*3+8+10, s, i->little, BLUE);
}

void draw_use_volume(INTERFACE *i, instrument *inst)
{
  /* up or down everything, not very fast, but what the heck ? it's much more simple,
   * and we don't spend our time in push/poping those buttons, do we ?
   */
  draw_3d_box_around(i->X_ecran, inst->pixmap, i->context, 3*5+2*6, 65*2+8+21+3+2+4, 10, 10, LIGHTBLUE,
		     inst->volume_type & 1 ? DOWN_3D : UP_3D);
  draw_3d_box_around(i->X_ecran, inst->pixmap, i->context, 3*5+2*6+24*6, 65*2+8+21+3+2+4, 10, 10, LIGHTBLUE,
		     inst->volume_type & 2 ? DOWN_3D : UP_3D);
  draw_3d_box_around(i->X_ecran, inst->pixmap, i->context, 3*5+2*6+40*6, 65*2+8+21+3+2+4, 10, 10, LIGHTBLUE,
		     inst->volume_type & 4 ? DOWN_3D : UP_3D);
}

void draw_nb_points_panning(INTERFACE *i, instrument *inst)
{
  char s[5];
  draw_box(i->X_ecran, inst->pixmap, i->context, NIX+9+6*7+5+13+3, 5+PINY+1+4+4+8, 6*6-4-16, 12, LIGHTBLUE);
  sprintf(s, "%2d", inst->nb_panning_points);
  draw_string(i->X_ecran, inst->pixmap, i->context, NIX+9+6*7+5+13+3+2, 5+PINY+1+4+4+8+10, s, i->little, BLUE);
}

void draw_sustain_point_panning(INTERFACE *i, instrument *inst)
{
  char s[5];
  draw_box(i->X_ecran, inst->pixmap, i->context, NIX+9+6*7+5+13+3, 5+PINY+1+4+4+13+3+13+6+8, 6*6-4-16, 12, LIGHTBLUE);
  sprintf(s, "%2d", inst->panning_sustain_point);
  draw_string(i->X_ecran, inst->pixmap, i->context, NIX+9+6*7+5+13+3+2, 5+PINY+1+4+4+13+3+13+6+8+10, s, i->little, BLUE);
}

void draw_loop_start_panning(INTERFACE *i, instrument *inst)
{
  char s[5];
  draw_box(i->X_ecran, inst->pixmap, i->context, NIX+9+6*7+5+13+3, 5+PINY+1+4+4+(13+3+13+6)*2+8, 6*6-4-16, 12, LIGHTBLUE);
  sprintf(s, "%2d", inst->panning_loop_start);
  draw_string(i->X_ecran, inst->pixmap, i->context, NIX+9+6*7+5+13+3+2, 5+PINY+1+4+4+(13+3+13+6)*2+18, s, i->little, BLUE);
}

void draw_loop_end_panning(INTERFACE *i, instrument *inst)
{
  char s[5];
  draw_box(i->X_ecran, inst->pixmap, i->context, NIX+9+6*7+5+13+3, 5+PINY+1+4+4+(13+3+13+6)*3+8, 6*6-4-16, 12, LIGHTBLUE);
  sprintf(s, "%2d", inst->panning_loop_end);
  draw_string(i->X_ecran, inst->pixmap, i->context, NIX+9+6*7+5+13+3+2, 5+PINY+1+4+4+(13+3+13+6)*3+18, s, i->little, BLUE);
}

void draw_use_panning(INTERFACE *i, instrument *inst)
{
  draw_3d_box_around(i->X_ecran, inst->pixmap, i->context, 3*5+2*6-3, PINY-2+65*2+8+21+3+2+4, 10, 10, LIGHTBLUE,
		     inst->panning_type & 1 ? DOWN_3D : UP_3D);
  draw_3d_box_around(i->X_ecran, inst->pixmap, i->context, 3*5+2*6+24*6+3, PINY-2+65*2+8+21+3+2+4, 10, 10, LIGHTBLUE,
		     inst->panning_type & 2 ? DOWN_3D : UP_3D);
  draw_3d_box_around(i->X_ecran, inst->pixmap, i->context, 3*5+2*6+40*6+3, PINY-2+65*2+8+21+3+2+4, 10, 10, LIGHTBLUE,
		     inst->panning_type & 4 ? DOWN_3D : UP_3D);
}

void draw_fadeout(INTERFACE *i, instrument *inst)
{
  char s[5];
  draw_box(i->X_ecran, inst->pixmap, i->context, VFX+9+6*7+5+13+3, 5+VFY+1+4+4+8, 7*6-4-16, 12, LIGHTBLUE);
  sprintf(s, "%3.3X", inst->volume_fadeout);
  draw_string(i->X_ecran, inst->pixmap, i->context, VFX+9+6*7+5+13+3+2, 5+VFY+1+4+4+8+10, s, i->little, BLUE);
}

void draw_vib_type(INTERFACE *i, instrument *inst)
{
  /* once again, we draw all, but once again it does consume too much cpu */
  draw_3d_box_around(i->X_ecran, inst->pixmap, i->context, VFX+50, VFY+74, 10, 10, LIGHTBLUE,
		     inst->vibrato_type==0 ? DOWN_3D : UP_3D);
  draw_3d_box_around(i->X_ecran, inst->pixmap, i->context, VFX+50, VFY+74+20, 10, 10, LIGHTBLUE,
		     inst->vibrato_type==1 ? DOWN_3D : UP_3D);
  draw_3d_box_around(i->X_ecran, inst->pixmap, i->context, VFX+50, VFY+74+20*2, 10, 10, LIGHTBLUE,
		     inst->vibrato_type==2 ? DOWN_3D : UP_3D);
  draw_3d_box_around(i->X_ecran, inst->pixmap, i->context, VFX+50, VFY+74+20*3, 10, 10, LIGHTBLUE,
		     inst->vibrato_type==3 ? DOWN_3D : UP_3D);
}

void draw_vib_speed(INTERFACE *i, instrument *inst)
{
  char s[5];
  draw_box(i->X_ecran, inst->pixmap, i->context, VFX+9+6*7+5+13+3, 5+VFY+1+4+4+(13+3+13+6)*4+8, 6*6-4-16, 12, LIGHTBLUE);
  sprintf(s, "%2.2X", inst->vibrato_rate);
  draw_string(i->X_ecran, inst->pixmap, i->context, VFX+9+6*7+5+13+3+2, 5+VFY+1+4+4+(13+3+13+6)*4+8+10, s, i->little, BLUE);
}

void draw_vib_sweep(INTERFACE *i, instrument *inst)
{
  char s[5];
  draw_box(i->X_ecran, inst->pixmap, i->context, VFX+9+6*7+5+13+3, 5+VFY+1+4+4+(13+3+13+6)*5+8, 6*6-4-16, 12, LIGHTBLUE);
  sprintf(s, "%2.2X", inst->vibrato_sweep);
  draw_string(i->X_ecran, inst->pixmap, i->context, VFX+9+6*7+5+13+3+2, 5+VFY+1+4+4+(13+3+13+6)*5+8+10, s, i->little, BLUE);
}

void draw_vib_depth(INTERFACE *i, instrument *inst)
{
  char s[5];
  draw_box(i->X_ecran, inst->pixmap, i->context, VFX+9+6*7+5+13+3, 5+VFY+1+4+4+(13+3+13+6)*6+8, 5*6-4-16, 12, LIGHTBLUE);
  sprintf(s, "%1.1X", inst->vibrato_depth);
  draw_string(i->X_ecran, inst->pixmap, i->context, VFX+9+6*7+5+13+3+2, 5+VFY+1+4+4+(13+3+13+6)*6+8+10, s, i->little, BLUE);
}

void draw_keyboard(INTERFACE *i, instrument *inst)
{
  int j;
  char s[5];
  static int is_white[12]={1, 0, 1, 0, 1, 0, 1, 1, 0, 1, 0, 1};
  static int inc_y1[12]={0, 0, 9, 0, 9*2, 0, 9*3, 9*4, 0, 9*5, 0, 9*6};
  static int inc_y2[12]={0, 4, 0, 4+9, 0, 4+9*2, 0, 0, 4+9*4, 0, 4+9*5, 0};
  int x, y;
  unsigned long bgcolor, fgcolor;
  for (j=0; j<8*12; j++) {
    if (is_white[j%12]) {
      /* a white key */
      x=(9*7*8+16+23-(9*8*7+8-1))/2+5+(j/12)*9*7-1+inc_y1[j%12];
      y=366+4+11-1;
      /* make the test here is bad, but easier to code (a good thing would to take it out of the for, but...) */
      if (inst->mode==EDIT && inst->subwindow==7 && inst->cur_pos_7==j) {
	bgcolor=RED;
	fgcolor=YELLOW;
      } else {
	bgcolor=WHITE;
	fgcolor=BLACK;
      }
    } else {
      x=(9*7*8+16+23-(9*8*7+8-1))/2+5+(j/12)*9*7+inc_y2[j%12];
      y=366+4+11-1-10;
      if (inst->mode==EDIT && inst->subwindow==7 && inst->cur_pos_7==j) {
	bgcolor=RED;
	fgcolor=YELLOW;
      } else {
	bgcolor=BLACK;
	fgcolor=WHITE;
      }
    }
    draw_box(i->X_ecran, inst->pixmap, i->context, x, y, 6+2, 7+3, bgcolor);
    sprintf(s, "%1.1X", inst->sample_number[j]);
    draw_string(i->X_ecran, inst->pixmap, i->context, x+1, y+9, s, i->little, fgcolor);
  }
}

/* draw all data */
void instrument_put_data(INTERFACE *i, instrument *inst)
{
  draw_volume_envelope(i, inst);
  draw_panning_envelope(i, inst);
  draw_nb_points_volume(i, inst);
  draw_sustain_point_volume(i, inst);
  draw_loop_start_volume(i, inst);
  draw_loop_end_volume(i, inst);
  draw_use_volume(i, inst);
  draw_nb_points_panning(i, inst);
  draw_sustain_point_panning(i, inst);
  draw_loop_start_panning(i, inst);
  draw_loop_end_panning(i, inst);
  draw_use_panning(i, inst);
  draw_fadeout(i, inst);
  draw_vib_type(i, inst);
  draw_vib_speed(i, inst);
  draw_vib_sweep(i, inst);
  draw_vib_depth(i, inst);
  draw_keyboard(i, inst);
}

static int inst_col[9][4]={
  {4, 4, 325+8+12, 65*2+8+12+3+3},
  {NIX+5, 5+4, 148-6*9-6, 74+71},
  {4+3*5, 65*2+8+21+3+2, 325+8+12-6*5, 18},
  {4, PINY+4, 325+8+12, 64*2+8+12+3+3},
  {NIX+5, 5+PINY+4, 148-6*9-6, 74+71},
  {4+3*5-3, PINY-2+65*2+8+21+3+2, 325+8+12-6*5+6, 18},
  {VFX+5, 5+VFY+4, 148-6*9, 74+71+(13+3+13+6)*3},
  {(9*7*8+16+23-(9*8*7+8-1))/2, 366, 9*8*7+8-1, 28},
  {470, 280, 8*5+4+3, 18}
};

void around_instrument(INTERFACE *i, instrument *inst, int type)
{
  unsigned long color=type==IN ? WHITE : BLACK;
  draw_rectangle(i->X_ecran, inst->pixmap, i->context, inst_col[inst->subwindow][0]-2, inst_col[inst->subwindow][1]-2,
		 inst_col[inst->subwindow][2]+4, inst_col[inst->subwindow][3]+4, 2, color);
  draw_rectangle(i->X_ecran, inst->window, i->context, inst_col[inst->subwindow][0]-2, inst_col[inst->subwindow][1]-2,
		 inst_col[inst->subwindow][2]+4, inst_col[inst->subwindow][3]+4, 2, color);
}

/* put the name of the window (i am not sure of this code, shoud read books to know if it's correct */
void put_name_inst(INTERFACE *i, XM *xm, instrument *inst)
{
  XTextProperty proper;
  char z[200];
  char *d=&z[0];
  int xn, in;
  for (xn=0; ;xn++)
    if (xm==i->xms[xn])
      break;
  for (in=0; ;in++)
    if (inst==xm->instruments[in])
      break;
  sprintf(z, "Xsoundtrack : instrument editor - Xm %2.2X - Instrument %2.2X", xn+1, in+1);
  if (!XStringListToTextProperty(&d, 1, &proper))
    fprintf(stderr, "instrument.put_name_inst error with XStringToTextProperty\n");
  else {
    XSetWMName(i->X_ecran, inst->window, &proper);
    XFree(proper.value);
  }
}

void in_out_instrument(INTERFACE *i, instrument *inst, int type)
{
  unsigned long color=inst->subwindow==8 ? LIGHTBLUE : GREY;
  draw_3d_box_around(i->X_ecran, inst->pixmap, i->context, inst_col[inst->subwindow][0], inst_col[inst->subwindow][1],
		     inst_col[inst->subwindow][2], inst_col[inst->subwindow][3], color, type);
  draw_3d_box_around(i->X_ecran, inst->window, i->context, inst_col[inst->subwindow][0], inst_col[inst->subwindow][1],
		     inst_col[inst->subwindow][2], inst_col[inst->subwindow][3], color, type);
}

/* this function creates the instrument-window associated with the current instrument */
/* return 0 if error 1 otherwise */
int init_instrument_interface(INTERFACE *i, XM *xm)
{
  instrument *inst;

  /* first of all, get the good instrument (create a header if necessary) */
  if (!(inst=xm->instruments[xm->start_5+xm->cur_pos_5])) {
    sample dum;
    int i;
    if(!(inst=xm->instruments[xm->start_5+xm->cur_pos_5]=(instrument *)malloc(sizeof(instrument))))
      return 0;
    memset(inst, 0, sizeof(instrument));
    memset(&dum, 0, sizeof(sample));
    dum.sample_volume=0x40;
    dum.sample_panning=0x80;
    for (i=0; i<16; i++)
      inst->sample_headers[i]=dum;
  }

  if (inst->is_shown) {
    XBell(i->X_ecran, 0);
    return 1;
  }

  /* create the menu window */
  #define HAUTEUR		(64*2+65*2+20*2+100)
  #define LARGEUR		(9*7*8+16+23)
  inst->window=XCreateSimpleWindow(i->X_ecran, DefaultRootWindow(i->X_ecran),
				   0, 0, LARGEUR, HAUTEUR, 3, 3,
				   BlackPixel(i->X_ecran, DefaultScreen(i->X_ecran)));
  XMapWindow(i->X_ecran, inst->window);
  XSelectInput(i->X_ecran, inst->window, KeyPressMask|ExposureMask);

  put_name_inst(i, xm, inst);

  /* the pixmap */
  inst->pixmap=XCreatePixmap(i->X_ecran, DefaultRootWindow(i->X_ecran), LARGEUR, HAUTEUR,
			     DefaultDepth(i->X_ecran, DefaultScreen(i->X_ecran)));
  draw_box(i->X_ecran, inst->pixmap, i->context, 0, 0, LARGEUR, HAUTEUR, BLACK);

  init_instrument_pixmap(i, inst);

  instrument_put_data(i, inst);

  /* init the vars */
  inst->is_shown=1;
  inst->mode=MOVING;
  inst->subwindow=0;
  inst->cur_pos_0=0;
  inst->cur_pos_1=0;
  inst->cur_pos_2=0;
  inst->cur_pos_3=0;
  inst->cur_pos_4=0;
  inst->cur_pos_5=0;
  inst->cur_pos_6=0;
  inst->cur_pos_7=0;
  inst->up_subwin_7=5;
  around_instrument(i, inst, IN);

  return 1;
}

void draw_inst(INTERFACE *i, instrument *inst)
{
  XCopyArea(i->X_ecran, inst->pixmap, inst->window, i->context, 0, 0, LARGEUR, HAUTEUR, 0, 0);
}

void other_instrument(INTERFACE *i, instrument *inst, KeySym ks)
{
  void interface_subwin_5(INTERFACE *, XM *, KeySym);
  extern XM *the_xm;	/* it is set by the event-dispatcher */
  instrument *to;

  interface_subwin_5(i, the_xm, ks);

  if (!(to=the_xm->instruments[the_xm->start_5+the_xm->cur_pos_5])) {
    sample dum;
    int l;
    if(!(to=the_xm->instruments[the_xm->start_5+the_xm->cur_pos_5]=(instrument *)malloc(sizeof(instrument)))) {
      perror("instrument.other_instrument malloc failed");
      XBell(i->X_ecran, 0);
      return;
    }
    memset(to, 0, sizeof(instrument));
    memset(&dum, 0, sizeof(sample));
    dum.sample_volume=0x40;
    dum.sample_panning=0x80;
    for (l=0; l<16; l++)
      to->sample_headers[l]=dum;
  }

  if (to==inst)
    return;

  to->window=inst->window;
  to->pixmap=inst->pixmap;
  inst->is_shown=0;
  to->is_shown=1;
  to->mode=MOVING;
  draw_box(i->X_ecran, to->pixmap, i->context, 0, 0, LARGEUR, HAUTEUR, BLACK);
  init_instrument_pixmap(i, to);
  instrument_put_data(i, to);
  to->subwindow=0;
  to->cur_pos_0=0;
  to->cur_pos_1=0;
  to->cur_pos_2=0;
  to->cur_pos_3=0;
  to->cur_pos_4=0;
  to->cur_pos_5=0;
  to->cur_pos_6=0;
  to->cur_pos_7=0;
  to->up_subwin_7=5;
  around_instrument(i, to, IN);
  draw_inst(i, to);
  put_name_inst(i, the_xm, to);
}

void keypress_instrument(INTERFACE *i, instrument *inst, XKeyEvent *event)
{
  extern unsigned int status;		/* re-use it, why not, huh ? */
  char dum[50];
  KeySym ks;

  status=event->state;

  XLookupString(event, dum, 50, &ks, (XComposeStatus *)0);

  if (ks==NoSymbol) {
    fprintf(stderr, "instrument.keypress_instrument : warning ks=NoSymbol\n");
    return;
  }

  if (ks==XK_Escape && inst->mode==EDIT) {
    inst->mode=MOVING;
    instrument_exit_subwin[inst->subwindow](i, inst);
    return in_out_instrument(i, inst, UP_3D);
  }

  if (inst->mode==EDIT)
    return instrument_subwin[inst->subwindow](i, inst, ks);

  /* once again, here come very nicies switches ! */
  switch(ks) {
    case XK_Return :
      inst->mode=EDIT;
      in_out_instrument(i, inst, DOWN_3D);
      instrument_enter_subwin[inst->subwindow](i, inst);
      break;
    case XK_Right :
      switch(inst->subwindow) {
	default :
	  break;
	case 0 :
	  around_instrument(i, inst, OUT);
	  inst->subwindow=1;
	  around_instrument(i, inst, IN);
	  break;
	case 1 :
	  around_instrument(i, inst, OUT);
	  inst->subwindow=6;
	  around_instrument(i, inst, IN);
	  break;
	case 2 :
	  around_instrument(i, inst, OUT);
	  inst->subwindow=1;
	  around_instrument(i, inst, IN);
	  break;
	case 3 :
	  around_instrument(i, inst, OUT);
	  inst->subwindow=4;
	  around_instrument(i, inst, IN);
	  break;
	case 4 :
	  around_instrument(i, inst, OUT);
	  inst->subwindow=8;
	  around_instrument(i, inst, IN);
	  break;
	case 5 :
	  around_instrument(i, inst, OUT);
	  inst->subwindow=4;
	  around_instrument(i, inst, IN);
	  break;
      }
      break;
    case XK_Left :
      switch(inst->subwindow) {
	default :
	  break;
	case 1 :
	  around_instrument(i, inst, OUT);
	  inst->subwindow=0;
	  around_instrument(i, inst, IN);
	  break;
	case 4 :
	  around_instrument(i, inst, OUT);
	  inst->subwindow=3;
	  around_instrument(i, inst, IN);
	  break;
	case 6 :
	  around_instrument(i, inst, OUT);
	  inst->subwindow=1;
	  around_instrument(i, inst, IN);
	  break;
	case 8 :
	  around_instrument(i, inst, OUT);
	  inst->subwindow=4;
	  around_instrument(i, inst, IN);
	  break;
      }
      break;
    case XK_Down :
      switch(inst->subwindow) {
	default :
	  break;
	case 0 :
	  around_instrument(i, inst, OUT);
	  inst->subwindow=2;
	  around_instrument(i, inst, IN);
	  break;
	case 1 :
	  around_instrument(i, inst, OUT);
	  inst->subwindow=4;
	  around_instrument(i, inst, IN);
	  break;
	case 2 :
	  around_instrument(i, inst, OUT);
	  inst->subwindow=3;
	  around_instrument(i, inst, IN);
	  break;
	case 3 :
	  around_instrument(i, inst, OUT);
	  inst->subwindow=5;
	  around_instrument(i, inst, IN);
	  break;
	case 4 :
	  around_instrument(i, inst, OUT);
	  inst->subwindow=7;
	  inst->up_subwin_7=4;
	  around_instrument(i, inst, IN);
	  break;
	case 5 :
	  around_instrument(i, inst, OUT);
	  inst->subwindow=7;
	  inst->up_subwin_7=5;
	  around_instrument(i, inst, IN);
	  break;
	case 6 :
	  around_instrument(i, inst, OUT);
	  inst->subwindow=8;
	  around_instrument(i, inst, IN);
	  break;
	case 8 :
	  around_instrument(i, inst, OUT);
	  inst->subwindow=7;
	  inst->up_subwin_7=8;
	  around_instrument(i, inst, IN);
	  break;
      }
      break;
    case XK_Up :
      switch(inst->subwindow) {
	default :
	  break;
	case 2 :
	  around_instrument(i, inst, OUT);
	  inst->subwindow=0;
	  around_instrument(i, inst, IN);
	  break;
	case 3 :
	  around_instrument(i, inst, OUT);
	  inst->subwindow=2;
	  around_instrument(i, inst, IN);
	  break;
	case 4 :
	  around_instrument(i, inst, OUT);
	  inst->subwindow=1;
	  around_instrument(i, inst, IN);
	  break;
	case 5 :
	  around_instrument(i, inst, OUT);
	  inst->subwindow=3;
	  around_instrument(i, inst, IN);
	  break;
	case 7 :
	  around_instrument(i, inst, OUT);
	  inst->subwindow=inst->up_subwin_7;
	  around_instrument(i, inst, IN);
	  break;
	case 8 :
	  around_instrument(i, inst, OUT);
	  inst->subwindow=6;
	  around_instrument(i, inst, IN);
	  break;
      }
      break;
      case XK_Page_Up : /* we wanna see previous instrument */
	other_instrument(i, inst, XK_Up);
	break;
      case XK_Page_Down : /* we wanna see next instrument */
	other_instrument(i, inst, XK_Down);
	break;
  }
}

void instrument_handle_event(INTERFACE *i, instrument *inst, XEvent *event)
{
  switch (event->type) {
    case Expose :
      draw_inst(i, inst);
      break;
    case KeyPress :
      keypress_instrument(i, inst, &event->xkey);
      break;
  }
}
