/*
 * This file is part of xsoundtrack - a sound tracker for Linux.
 * Sed - july august 1998.
 * This is total free software.
 *-------------------------------------
 * the interface main file.
 */

/* a little main-test */
#include <stdio.h>
#include <X11/Xlib.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#include <errno.h>

/* includes from ../player */
#include "card.h"
#include "xm.h"
#include "mixer.h"
/* local includes */
#include "interface.h"

/* return -1 on error, 0 if soundcard event is there, 1 if X event is here */
int wait_event(INTERFACE *i)
{
  int fd=get_sound_fd();
  fd_set r, w, e;

  FD_ZERO(&r);
  FD_ZERO(&w);
  FD_ZERO(&e);

  if (i->current_xm && i->current_xm->mix && (!i->current_xm->do_we_dump))
    FD_SET(fd, &w);
  else
    if (i->in_record)
      FD_SET(fd, &r);

  FD_SET(ConnectionNumber(i->X_ecran), &r);

  /* before to select, flush X */
  //XFlush(i->X_ecran);

  while (select(FD_SETSIZE, &r, &w, &e, 0)==-1) {
    if (errno!=EINTR) {
      perror("select");
      return -1;
    }
  }

  if (i->current_xm && i->current_xm->mix && (!i->current_xm->do_we_dump)) {
    if (FD_ISSET(fd, &w)) {
      return 0;
    }
  } else {
    if (i->in_record)
      if (FD_ISSET(fd, &r)) {
	return 0;
      }
  }

  if (FD_ISSET(ConnectionNumber(i->X_ecran), &r))
    return 1;

  fprintf(stderr, "main.real_main: no sound nor X descriptor are set, impossible\n");
  return -1;
}

/* not a good idea to call it real_main, but have no other in mind */
/* ok, with select, this function simply does one select over the sound descriptor
 * if mixing and the X descriptor, then it returns (that's what it did before)
 * (don't forget to deal with recording). */
void real_main(INTERFACE *i)
{
  int ev;

  void draw_patterns(INTERFACE *, XM *);
  void draw_patterns_order(INTERFACE *, XM *);
  void draw_default_bpm(INTERFACE *, XM *);
  void draw_default_speed(INTERFACE *, XM *);

  ev=wait_event(i);
  if (ev==1) {
    handle_interface_event(i);
    return;
  }
  if (ev==-1)
    return;
  /* here it's a soundcard event (read or write is possible, depending if we are in read or write mode */
  /* if read, then it's handled somewhere else, only deal with write */
  if (i->current_xm && i->current_xm->mix) {
    mix(i->current_xm);
#if 1
    if (i->current_xm) {
      if (i->current_xm->new_line) {
	i->current_xm->new_line=0;
	draw_patterns(i, i->current_xm);
      }
      if (i->current_xm->new_pattern) {
	i->current_xm->new_pattern=0;
	draw_patterns_order(i, i->current_xm);
      }
      if (i->current_xm->new_tempo) {
	i->current_xm->new_tempo=0;
	draw_default_bpm(i, i->current_xm);
	draw_default_speed(i, i->current_xm);
      }
    }
#endif
  }
}

int main(int nb, char **arg)
{
  INTERFACE interface;

  if (!init_interface(&interface)) {
    fprintf(stderr, "Error while initing the interface (are you under X-window ?)\n");
    exit(1);
  }

  if (!init_mixer(44100, 16, 1)) {
    fprintf(stderr, "error with initing the soft-mixer, bye bye\n");
    exit(1);
  }

  while(1)
    real_main(&interface);

  close_sound();
  close_interface(&interface);

  return 0;
}
